import React from 'react'
import { isEmpty } from 'lodash'
import { connect } from 'react-redux'
import { reduxForm } from 'redux-form'
import { required } from 'xpub-validators'
import { Button, H2, TextField, ValidatedField } from '@pubsweet/ui'
import {
  compose,
  withState,
  lifecycle,
  withHandlers,
  setDisplayName,
} from 'recompose'
import {
  Row,
  Text,
  Item,
  Label,
  OpenModal,
  ShadowedBox,
} from 'pubsweet-component-faraday-ui'

import {
  digitValidator,
  parseSearchParams,
  manuscriptIdSizeValidator,
} from '../utils'
import {
  technicalDecision,
  technicalCheckFetching,
} from '../../redux/technicalCheck'

const sizeValidator = manuscriptIdSizeValidator(7)

const Enhanched = () => (
  <Row>
    <Item vertical>
      <Label required>Manuscript ID</Label>
      <ValidatedField
        component={TextField}
        name="customId"
        validate={[required, digitValidator, sizeValidator]}
      />
    </Item>
  </Row>
)

const FormModal = reduxForm({
  form: 'eqs',
  onSubmit: (
    { customId },
    dispatch,
    { technicalDecision, params: { collectionId, token }, setSuccess },
  ) => ({ hideModal, setModalError }) => {
    technicalDecision({
      token,
      customId,
      step: 'eqs',
      agree: true,
      collectionId,
    })
      .then(() => {
        hideModal()
        setSuccess(`Manuscript accepted. Thank you for your technical check!`)
      })
      .catch(e => {
        setModalError(`There was an error: ${e}`)
      })
  },
})(({ isFetching, handleSubmit, technicalDecision, ...formProps }) => (
  <OpenModal
    content={Enhanched}
    isFetching={isFetching}
    modalKey="acceptManuscript"
    onConfirm={modalProps => handleSubmit()(modalProps)}
    technicalDecision={technicalDecision}
    title="Accept Manuscript"
  >
    {showModal => (
      <Button onClick={showModal} primary>
        YES
      </Button>
    )}
  </OpenModal>
))

const EQSDecisionPage = ({
  params,
  isFetching,
  eqsDecision,
  successMessage,
  rejectManuscript,
  technicalDecision,
  ...rest
}) => (
  <ShadowedBox center mt={5}>
    <H2>Editorial decision</H2>
    {!successMessage && (
      <Row mt={2}>
        <Text secondary>
          Did manuscript titled <b>{params.title}</b> pass EQS checks?
        </Text>
      </Row>
    )}
    {successMessage && (
      <Row mt={1}>
        <Text>{successMessage}</Text>
      </Row>
    )}
    {isEmpty(successMessage) && (
      <Row justify="space-around" mt={2}>
        <OpenModal
          isFetching={isFetching}
          modalKey="denyManuscript"
          onConfirm={rejectManuscript}
          subtitle="Are you sure you want to reject this manuscript?"
          title="Reject manuscript"
        >
          {showModal => <Button onClick={showModal}>NO</Button>}
        </OpenModal>
        <FormModal
          isFetching={isFetching}
          params={params}
          technicalDecision={technicalDecision}
          {...rest}
        />
      </Row>
    )}
  </ShadowedBox>
)

export default compose(
  setDisplayName('EQS Decision page'),
  connect(
    (state, { params }) => ({
      isFetching: technicalCheckFetching(state),
    }),
    { technicalDecision },
  ),
  withState('params', 'setParams', {
    title: '',
    token: null,
    customId: null,
    collectionId: null,
  }),
  withState('successMessage', 'setSuccess', ''),
  lifecycle({
    componentDidMount() {
      const { location, setParams } = this.props
      const { customId, collectionId, token, title } = parseSearchParams(
        location.search,
      )
      setParams({ customId, collectionId, token, title })
    },
  }),
  withHandlers({
    rejectManuscript: ({
      setSuccess,
      technicalDecision,
      params: { collectionId, token },
    }) => ({ hideModal, setModalError }) => {
      technicalDecision({
        token,
        step: 'eqs',
        agree: false,
        collectionId,
      })
        .then(() => {
          hideModal()
          setSuccess(`Manuscript rejected. Thank you for your technical check!`)
        })
        .catch(e => {
          setModalError(`There was an error: ${e}`)
        })
    },
  }),
)(EQSDecisionPage)

// #region styles
// #endregion
